/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  boot.c
 * Purpose: Perform the actual booting process
 * Entries: do_boot
 *
 **************************************************************************
 *
 * Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../headers/net.h"
#include "../headers/boot.h"
#include "../headers/romlib.h"
#include "../headers/general.h"
#include "./bootpriv.h"
#include "./bootp.h"
#include "./load.h"


/*
 **************************************************************************
 *
 * Miscellaneous defines:
 */
#define INPUT_TIMEOUT	36	/* wait this many ticks for user input	*/
#define CHR_EXIT	'x'	/* character to abort boot process	*/



/*
 **************************************************************************
 * 
 * Print an IP number.
 */
static void printip(t_ipaddr ip)
{
  unsigned char *cp = &(unsigned char)ip;

  printf("%u.%u.%u.%u", cp[0], cp[1], cp[2], cp[3]);
}



/*
 **************************************************************************
 * 
 * Actually perform the booting process by first calling the BOOTP client,
 * and then loading the operating system using TFTP.
 *
 */
void do_boot(void)
{
  struct bootp *bp;

  /* Get BOOTP record from server */
  if ((bp = bootp()) == NULL)
	return;

  /* Let the user know what we got */
  printf("\n\nLocal IP:  "); printip(ntohl(bp->bp_yiaddr));
  printf("\nServer IP: "); printip(ntohl(bp->bp_siaddr));
  printf(" (%s)\n\n\n", bp->bp_sname);

  /* Check if the user really wants to boot. */
  set_timeout(INPUT_TIMEOUT);
  while (!chk_timeout() && !checkkey());
  if (checkkey() && getkey() == CHR_EXIT)
	fatal();

  /* Get the boot file from the server */
  if (!load(ntohl(bp->bp_siaddr), (char *)(bp->bp_file), sizeof(bp->bp_file)))
	return;

  /* Finally call the new boot image */
  call_image(bp);
}

