;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  basicio.asm
;* Purpose: Basic I/O functions for DOS simulator
;* Entries: dos01, dos02, dos03, dos04, dos05, dos06, dos07, dos08, dos09,
;*          dos0A, dos0B, dos0C
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include ..\headers\asm\minidos.inc
include .\dospriv.inc


;
;**************************************************************************
;
; Serial ports and printer ports are usually not necessary for a boot
; rom, so setup some defines here, which regulate assembly of aux and
; prn code.
;
NEEDCOM = 0			; we don't need any of these
NEEDPRN = 0
NEEDOPEN = 0

ifndef CONONLY
NEEDCOM = 1
NEEDPRN = 1			; setup everything if requested
NEEDOPEN = 1
else
ifdef NEEDFILEIO
NEEDOPEN = 1			; if file I/O is required, then we have
endif				; to include the open routine
endif


;
;**************************************************************************
;
; Miscalleneous equates
;
PRN_PORT	equ	0		; PRN is printer port 0
AUX_PORT	equ	0		; AUX is serial port 0
AUX_INIT	equ	10100011b	; AUX port initialization value:
					;  2400 Baud, no parity, 1 stop bit,
					;  8 data bits
CHR_BS		equ	08h		; Backspace character
CHR_RET		equ	0Dh		; Return character
CHR_LF		equ	0Ah		; Line-feed character
CHR_FF		equ	0Ch		; Form-feed character
CHR_TAB		equ	09h		; Tab character


;
;**************************************************************************
;
; Data segment
;
data_start

		extrn	old_ds:word	; caller's DS register

if NEEDOPEN gt 0
auxflag		db	0		; flag if serial port initialized
prnflag		db	0		; flag if printer port initialized
endif

data_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	dos01, dos02, dos03	; define entry points
	public	dos04, dos05, dos06
	public	dos07, dos08, dos09
	public	dos0A, dos0B, dos0C

	public	devinit


;
;**************************************************************************
;
; Read character from keyboard and then send it to the screen
; Input:  none
; Output: AL  -  character
; Registers changed: AL
;
dos08		label	near
dos07		label	near
dos01		proc	near

	push	bx
	mov	bh,ah			; save AH
	call	keywait			; wait for character from keyboard
	mov	bl,al
	cmp	bh,1
	ja	dos011
	call	prnchr			; print character
dos011:	mov	ax,bx			; restore AH
	pop	bx
	ret

dos01		endp


;
;**************************************************************************
;
; Print character onto screen
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos02		proc	near

	push	ax
	mov	al,dl
	call	prnchr			; print character
	pop	ax
	ret

dos02		endp


;
;**************************************************************************
;
; Receive character on serial port COM1
; Input:  none
; Output: AL  -  character
; Registers changed: AL
;
dos03		proc	near

if NEEDCOM gt 0
	push	bx
	push	dx
	mov	bh,ah
	mov	ah,DEV_AUX
	mov	al,AUX_PORT
	call	devinit			; initialize serial device
	mov	ah,02h
	int	14h			; receive character
	mov	ah,bh			; restore AH
	pop	dx
	pop	bx
endif

	ret

dos03		endp


;
;**************************************************************************
;
; Send character to serial port COM1
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos04		proc	near

if NEEDCOM gt 0
	push	ax
	push	dx
	mov	ah,DEV_AUX
	mov	al,AUX_PORT
	call	devinit			; initialize serial device
	pop	ax
	push	ax			; restore character
	mov	ah,01h
	int	14h			; send character
	pop	dx
	pop	ax
endif

	ret

dos04		endp


;
;**************************************************************************
;
; Send character to printer port
; Input:  DL  -  character
; Output: none
; Registers changed: none
;
dos05		proc	near

if NEEDPRN gt 0
	push	ax
	push	dx
	mov	ah,DEV_PRN
	mov	al,PRN_PORT
	call	devinit			; initialize parallel port
	pop	ax
	push	ax			; restore character
	xor	ah,ah
	int	17h			; send character
	pop	dx
	pop	ax
endif

	ret

dos05		endp


;
;**************************************************************************
;
; Direct console I/O
; Input:  DL  -  character for console output, or 0FFh for console input
; Output: AL  -  character from console input, or 0 for console output
; Changed registers: AL
;
dos06		proc	near

	cmp	dl,0FFh			; check for function
	je	dos061
	xor	al,al
	jmp	dos02			; print character

dos061:	push	bx
	mov	bh,ah			; save AH
	mov	ah,01h
	int	16h			; get character from keyboard
	jz	dos062			; got no character
	xor	ah,ah
	int	16h			; get character from buffer
	or	al,al
	jnz	dos063

dos062:	xor	al,al			; set return value 0
dos063:	mov	ah,bh			; restore AH
	pop	bx
	ret

dos06		endp


;
;**************************************************************************
;
; Print character string onto screen. String is terminated by '$' character.
; Input:  DS:DX  -  pointer to string
; Output: none
; Registers changed: none
;
dos09		proc	near

	push	ax
	push	si
	push	ds
	mov	ds,old_ds		; get caller's DS
	mov	si,dx
dos091:	lodsb				; get next character
	cmp	al,'$'			; end of string?
	je	dos092
	call	prnchr			; print character
	jmp	dos091			; proceed with next character

dos092:	pop	ds
	pop	si
	pop	ax
	ret

dos09		endp


;
;**************************************************************************
;
; Read string from keyboard. The first character of the buffer contains
; the buffer size, the next character will receive the actual number
; of characters read.
; Input:  DS:DX  -  pointer to buffer
; Output: none
; Registers changed: none
;
dos0A		proc	near

	push	es
	push	ax
	push	bx
	push	di
	mov	es,old_ds		; ES:DI - pointer to buffer
	mov	di,dx
	mov	bx,2			; BX - offset into buffer

dos0A1:	mov	ah,es:[di]		; get length of buffer
	add	ah,2			; count for beginning two bytes
	cmp	bl,ah			; end of buffer reached?
	jae	dos0A4
	call	keywait			; get next character

	cmp	al,CHR_BS		; backspace?
	jne	dos0A2
	cmp	bx,2			; don't delete past beginning of
	jbe	dos0A1			; buffer
	dec	bx			; kill character in the buffer and
	jmp	short dos0A5		; on the screen

dos0A2:	cmp	al,CHR_RET		; carriage return?
	je	dos0A3
	mov	es:[di+bx],al		; save character
	inc	bx
dos0A5:	call	prnchr			; and print it
	jmp	dos0A1

dos0A3:	call	prnchr
	mov	al,CHR_LF		; print CR/LF
	call	prnchr
dos0A4:	sub	bx,2			; compute length of input string
	mov	es:[di+1],bl		; and save it in the buffer
	pop	di
	pop	bx
	pop	ax
	pop	es
	ret

dos0A		endp


;
;**************************************************************************
;
; Get keyboard status
; Input:  none
; Output: AL  -  0FFh -> key pressed, 000h -> no key pressed
; Changed registers: AL
;
dos0B		proc	near

	push	ax
	mov	ah,01h
	int	16h			; check BIOS keyboard buffer
	pop	ax
	jnz	dos0B1
	xor	al,al			; no character in buffer
	jmp	short dos0B2

dos0B1:	or	al,al			; don't count extended characters
	jz	dos0B2
	mov	al,0FFh			; yes, we have a character available
	or	al,al
dos0B2:	ret

dos0B		endp


;
;**************************************************************************
;
; Clear input buffer and then call input routine.
; Input:  AL  -  input function number
; Output: AL  -  input character
; Registers changed: depending on input function
;
dos0C		proc	near

	push	ax
dos0C1:	mov	ah,01h
	int	16h			; get keyboard status
	jz	dos0C2			; no more characters in buffer
	xor	ah,ah
	int	16h			; get character from input buffer
	jmp	dos0C1			; repeat until buffer empty
dos0C2:	pop	ax

	cmp	al,01h
	je	dos0C3
	cmp	al,07h
	je	dos0C3
	cmp	al,08h
	jne	dos0C4
dos0C3:	mov	ah,al
	jmp	dos01			; call DOS function 01

dos0C4:	cmp	al,06h
	jne	dos0C5
	jmp	dos06			; call DOS function 06
dos0C5:	cmp	al,0Ah
	jne	dos0C6
	jmp	dos0A			; call DOS function 0A
dos0C6:	xor	al,al
	ret				; no function to call

dos0C		endp


;
;**************************************************************************
;
; Wait for a key press
; Input:  none
; Output: AL  -  character
; Registers changed: AX
;
keywait		proc	near

keyw1:	xor	ah,ah
	int	16h			; get character from keyboard
	or	al,al			; check for extended keyboard character
	jz	keyw1
keyw9:	ret

keywait		endp


;
;**************************************************************************
;
; Print character onto console
; Input:  AL  -  character to print
; Output: none
; Registers changed: AX
;
prnchr		proc	near

	push	bx
	cmp	al,CHR_FF		; form feed character is special
	je	prnch1
	cmp	al,CHR_TAB		; tabulator is special as well
	je	prnch2
	mov	bx,0007h		; set color and screen page
	mov	ah,0Eh
	int	10h			; print character
	jmp	short prnch9

prnch1:	push	cx
	push	dx
	mov	ax,0600h		; clear screen
	mov	bx,0700h		; set color
	xor	cx,cx			; upper left corner
	mov	dx,0FFFFh		; lower right corner
	int	10h
	xor	bx,bx
	mov	ah,02h			; put cursor into upper left corner
	int	10h
	jmp	short prnch4

prnch2:	push	cx
	push	dx
	mov	ah,03h			; read cursor position
	xor	bh,bh
	int	10h
	and	dl,7
	mov	cx,8			; compute number of spaces to print
	sub	cl,dl
	jnz	prnch3
	mov	cx,8
prnch3:	mov	ax,0E20h		; print spaces
	mov	bx,0007h
	int	10h
	loop	prnch3
prnch4:	pop	dx
	pop	cx
prnch9:	pop	bx
	ret

prnchr		endp


;
;**************************************************************************
;
; Initialize a device.
; Input:  AH  -  device type (CON, AUX, PRN) - major number
;         AL  -  device number               - minor number
; Output: DX  -  minor device number
; Registers changed: AX, DX
;
devinit		proc	near

if NEEDOPEN gt 0
	push	cx
	mov	dl,al
	xor	dh,dh			; setup minor device number
	mov	cl,al
	mov	al,01h
	or	cl,cl			; compute device flag mask
	jz	devin1
	shl	al,cl
devin1:	pop	cx
	cmp	ah,DEV_CON		; console doesn't have to be
	je	devin9			; intialized.

; Initialize serial port

	cmp	ah,DEV_AUX
	jne	devin2
	test	auxflag,al		; already initialized?
	jnz	devin9
	or	auxflag,al		; set init flag
	mov	al,AUX_INIT
	xor	ah,ah
	int	14h			; initialize serial port
	jmp	short devin9

; Initialize parallel port

devin2:	cmp	ah,DEV_PRN
	jne	devin9
	test	prnflag,al		; already initialized?
	jnz	devin9
	or	prnflag,al		; set init flag
	mov	dl,cl
	xor	dh,dh
	mov	ah,01h
	int	17h			; initialize printer port
endif

devin9:	ret

devinit		endp


;
;**************************************************************************
;
text_end

	end

