;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  dospriv.inc
;* Purpose: Include file to define DOS and BIOS data structures
;* Entries: None
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Fatal error codes:
;
FATAL_NOMEM	equ	1		; not enough basic memory
FATAL_STACK	equ	2		; stack overflow error
FATAL_REENTRY	equ	3		; reentrant calling of int21h
FATAL_NOPROC	equ	4		; terminating non-active process
FATAL_MEMORY	equ	5		; memory corrupt


;
;**************************************************************************
;
; Definitions for memory management:
;
; A memory control block precedes every memory block in the lower ram
; area (e.g. from the end of the boot rom data area to the beginning
; of the operating system loading area). If the owner address is 0,
; the memory block is free.
;
mcb		struc

mcb_id		db	?		; ID of control block (see below)
mcb_owner	dw	?		; pointer to PSP of owner process
mcb_size	dw	?		; size in paragraphs

mcb		ends

; MCB id codes:

MCB_MEM_ID	equ	'M'		; indicates valid memory block
MCB_END_ID	equ	'Z'		; indicates last memory block


;
;**************************************************************************
;
; Definition of file handles. The first 5 file handles are reserved. The
; remaining file handles are built by using the lower two bits as the
; minor device code (which gives 4 minor devices), and the upper bits as
; the major device code (see below). Therefore the file handle will be
; computed as:
;
;    handle = (major * 4) + minor + RES_HANDLES
;
; This gives the number 16 as the maximum file handle. Of course this
; is not the way the real DOS uses file handles, but we are not using
; real files in with the simulator but only devices, so this approach
; saves a lot of unnecessary management code.
;
; Device codes (do not confuse with handle numbers!):

DEV_CON		equ	0		; console
DEV_AUX		equ	1		; serial port (0-3)
DEV_PRN		equ	2		; printer port (0-3)

MAX_HANDLE	equ	16		; maximum file handle value
MAX_MINOR	equ	3		; maximum minor device number


;
;**************************************************************************
;
; Definition of Program Segment Prefix (PSP):
;
psp		struc

		db	0CDh,020h	; OP-codes for INT20h
psp_endmem	dw	?		; first allocatable seg after program
		db	00h		; reserved
		db	09Ah		; far call OP-code to INT21h handler
psp_call21	dd	?		; pointer to INT21h handler
psp_int22	dd	?			; saved interrupt 22h vector
psp_int23	dd	?			; saved interrupt 23h vector
psp_int24	dd	?			; saved interrupt 24h vector
psp_parent	dw	?			; PSP of parent process
psp_handles	db	RES_HANDLES dup (0FFh)	; reserved file handles
		db	20-RES_HANDLES dup (0)	; file handle table
psp_env		dw	?			; segment of program environment
psp_oldstack	dd	?			; stack pointer of parent
psp_numhndl	dw	psp_env - psp_handles	; number of file handles
psp_ofshndl	dw	psp_handles		; offset of file handle table
psp_seghndl	dw	?			; segment of file handle table
		db	24 dup (00h)		; reserved
		db	0CDh,021h,0CBh		; OP-codes for INT21h and RETF
		db	9 dup (00h)		; reserved
psp_fcb1	db	16 dup (?)	; first part of FCB 1
psp_fcb2	db	16 dup (?)	; first part of FCB 2
		db	4 dup (00h)	; reserved
psp_cmdlsize	db	?		; length of command line
psp_cmdl	db	?		; command line (the first of 127 bytes)

psp		ends

PSP_MAXCMDL	equ	125		; maximum length of command line
PSP_SIZE	equ	256		; size of PSP with full command line

