;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  macros.inc
;* Purpose: Miscellanous assembler macros
;* Entries: None
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Make segment definitions easy and unique by using the following macros.
; Note that these macros have to be identical with the definitions in
; the segment layout include file.
;
; Macros for defining the floppy loader text segment:
;
load_start	macro
load_text	segment	byte public 'CODE'
		assume	cs:load_text
		assume	ds:load_text
		endm

load_end	macro
load_text	ends
		endm

;
; Macros for defining the general text segment:
;
text_start	macro
_text		segment	byte public 'CODE'
		assume	cs:cgroup
		assume	ds:dgroup
		endm

text_end	macro
_text		ends
		endm

;
; Macros for defining the constant data segment:
;
const_start	macro
_const		segment	word public 'CONST'
		endm

const_end	macro
_const		ends
		endm

;
; Macros for defining the general data segment:
;
data_start	macro
_data		segment word public 'DATA'
		endm

data_end	macro
_data		ends
		endm

;
; Macros for defining the BSS segment:
;
bss_start	macro
_bss		segment	word public 'BSS'
		endm

bss_end		macro
_bss		ends
		endm


;
;**************************************************************************
;
; Macros for making the high level C programming interface easier.
;
; Get an argument from the stack. This assumes the tiny model with a
; standard stack frame.
;
getarg		macro	reg,argnum

if ((.type &argnum) and 04h) eq 04h
		;; argnum is a constant value
		mov	&reg,[bp + 4 + (&argnum * 2)]
else
		;; argnum is a variable or register
  ifdifi <&reg>,<si>
		push	si
  endif
  ifdifi <&argnum>,<si>
		mov	si,&argnum
  endif
		shl	si,1
		mov	&reg,[bp + 4 + si]
  ifdifi <&reg>,<si>
		pop	si
  endif
endif

		endm

