/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  main.c
 * Purpose: Call all initialization routines and then start boot process
 * Entries: main
 *
 **************************************************************************
 *
 * Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../headers/minidos.h"
#include "../headers/packet.h"
#include "../headers/net.h"
#include "../headers/boot.h"
#include "../headers/romlib.h"
#include "../headers/general.h"


/*
 **************************************************************************
 * 
 * Main routine: call all the initialization routines, and then start
 * the net boot process
 *
 */
main(pktadr, pktarg, dataend)
void far *pktadr;
void far *pktarg;
void far *dataend;
{
  /* Initialize all modules */
  init_minidos(dataend);
  init_packet(pktadr, pktarg);
  init_net();

  /* Initialize the screen */
  printf("\nFree memory: %d\n", memavail());
  sleep(25);
  if (checkkey() && getkey() == CHR_STOP)
	(void)getkey();
  printf("\f\nBoot-Rom V%d.%d, (C) G. Kuhlmann\n\n", MAJOR_VER, MINOR_VER);

  /* Now actually load the operating system */
  do_boot();

  /* Oops, this should never happen */
  fatal();
}
