;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  fatal.asm
;* Purpose: Handle fatal errors, and system console messages
;* Entries: _fatal, prnstr, prnword, prnbyte
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include .\libpriv.inc


;
;**************************************************************************
;
; Define BIOS entry point for rebooting the system.
;
reboot	segment	at 0FFFFh

	org	0
doboot	label	far

reboot	ends



;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_fatal			; define entry points
	public	prnstr
	public	prnword
	public	prnbyte


;
;**************************************************************************
;
; Handle fatal errors
; Input:  none
; Output: none
;
_fatal		proc	near

	mov	bx,offset cgroup:fatmsg
	call	prnstr			; print startup message
	xor	ax,ax
	int	16h			; wait for key press

	cli
	cld
	xor	ax,ax
	mov	es,ax
	mov	ax,OLDINTS		; restore old interrupt table
	mov	ds,ax
	xor	si,si
	xor	di,di
	mov	cx,VECTSIZE/2
	rep	movsw

	mov	ax,0040h
	mov	ds,ax
	mov	ds:[0072h],1234h	; indicates warm boot
	jmp	doboot			; reboot the system

; Error messages

fatmsg	db	0Dh,0Ah,0Dh,0Ah
	db	'Press a key to reboot...'
	db	0

_fatal		endp


;
;**************************************************************************
;
; Print a string onto the console screen.
; Input:  CS:BX  -  pointer to zero terminated string
; Output: none
; Registers changed: AX, BX
;
prnstr		proc	near

prnst1:	mov	al,cs:[bx]
	or	al,al			; end of string
	jz	prnst2
	call	prnchar			; print character
	inc	bx			; proceed with next character
	jmp	prnst1
prnst2:	ret

prnstr		endp


;
;**************************************************************************
;
; Print a word/byte/nibble onto the console screen
; Input:  AX  -  value
; Output: none
; Registers changed: AX
;
prnword		proc	near

	push	ax
	mov	al,ah
	call	prnbyte			; print high word
	pop	ax			; print low word

prnbyte		label	near

	push	ax
	shr	al,1
	shr	al,1			; print high nibble
	shr	al,1
	shr	al,1
	call	prnnibble
	pop	ax			; print low nibble

prnnibble	label	near

	and	al,0Fh
	add	al,'0'			; convert into ASCII character
	cmp	al,'9'
	jbe	prnchar
	add	al,7

prnchar		label	near

	push	bx
	mov	bx,0007h		; color and screen page
	mov	ah,0Eh
	int	10h			; call BIOS to print character
	pop	bx
	ret

prnword		endp


;
;**************************************************************************
;
text_end

	end

