;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  keyboard.asm
;* Purpose: Some keyboard related functions
;* Entries: _getkey, _checkkey
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include .\libpriv.inc


;
;**************************************************************************
;
; BSS segment:
;
bss_start

keybuf		db	0		; temporary key buffer

bss_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_getkey			; define entry points
	public	_checkkey

	extrn	_dopoll:near


;
;**************************************************************************
;
; Wait for keyboard input, and return the key pressed. This routine
; does not accept extended key codes, and does not print anything.
; Input:  none
; Output: character representing pressed key
;
_getkey		proc	near

	xor	al,al
	xchg	al,keybuf		; first check keyboard buffer
	or	al,al
	jnz	getk9

getk1:	call	_dopoll			; care for polling events
	mov	ah,06h
	mov	dl,0FFh			; check DOS keyboard status
	int	21h
	jz	getk1			; no key pressed
	or	al,al			; check for extended key code
	jnz	getk9
	mov	ah,08h
	int	21h			; get extended key code
	jmp	getk1			; and discard it
getk9:	xor	ah,ah			; prepare return value
	ret

_getkey		endp


;
;**************************************************************************
;
; Check if there is a key code in the input buffer. This routine does
; not accept extended key codes.
; Input:  none
; Output: zero if buffer empty, otherwise non-zero.
;
_checkkey	proc

	mov	al,keybuf
	or	al,al			; first check keyboard buffer
	jnz	chkk8

chkk1:	call	_dopoll			; care for polling events
	mov	ah,06h
	mov	dl,0FFh			; check DOS keyboard status
	int	21h
	jz	chkk7
	or	al,al			; is it an extended key code?
	jnz	chkk8
	mov	ah,08h
	int	21h			; discard extended key code
	jmp	chkk1

chkk7:	xor	ax,ax			; return FALSE
	jmp	short chkk9

chkk8:	mov	ax,1			; return TRUE
chkk9:	ret

_checkkey	endp


;
;**************************************************************************
;
text_end

	end

