;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  misc.asm
;* Purpose: Routines for different things
;* Entries: _segds, _segcs, _random
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include .\libpriv.inc


;
;**************************************************************************
;
; Definitions for the random number generator:
;
RANDLOW		equ	4E35h		; Low multiplication value
RANDHIGH	equ	015Ah		; High multiplication value
RANDADD		equ	23968


;
;**************************************************************************
;
; BSS segment:
;
bss_start

rseed		dd	0		; seed for random number

bss_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_segcs			; define entry points
	public	_segds
	public	_random

	extrn	_get_ticks:near


;
;**************************************************************************
;
; Return value of DS segment register
; Input:  none
; Output: value of DS
;
_segds		proc	near

	mov	ax,ds
	ret

_segds		endp


;
;**************************************************************************
;
; Return value of CS segment register
; Input:  none
; Output: value of CS
;
_segcs		proc	near

	mov	ax,cs
	ret

_segcs		endp


;
;**************************************************************************
;
; Return a short integer random number
; Input:  none
; Output: random number
;
_random		proc	near

	mov	dx,word ptr [rseed+0]
	mov	ax,dx			; check if the seed has already been
	or	dx,word ptr [rseed+2]	; set
	jnz	rand1
	call	_get_ticks		; if not set it by reading the timer
	xchg	ah,dl			; tick count
	xchg	ah,dh			; mix the values a little bit
	xchg	ah,al
	mov 	word ptr [rseed+0],ax
	mov	word ptr [rseed+2],dx

rand1:	mov	dx,RANDHIGH
	mul	dx			; multiply the seed with a magic number
	mov	bx,ax
	mov	ax,RANDLOW
	mul	word ptr [rseed+0]
	mov	cx,ax
	add	bx,dx
	mov	ax,RANDLOW
	mul	word ptr [rseed+2]
	add	bx,ax
	add	cx,RANDADD		; add another magic number
	adc	bx,0
	mov	word ptr [rseed+0],cx	; save new seed number
	mov	word ptr [rseed+2],bx
	mov	ax,bx			; prepare return value
	and	ax,7FFFh		; make it a signed short integer
	ret

_random		endp


;
;**************************************************************************
;
text_end

	end

