;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  string.asm
;* Purpose: String and memory handling functions
;* Entries: _memcpy, _memcmp, _memset
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include .\libpriv.inc


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_memcpy			; define entry points
	public	_memcmp
	public	_memset


;
;**************************************************************************
;
; Copy memory from second arg to first arg.
; Input:  1. arg  -  far pointer to destination
;         2. arg  -  far pointer to source
;         3. arg  -  number of bytes to copy
; Output: none
;
_memcpy		proc	near

	cld
	push	bp
	mov	bp,sp			; setup standard stack frame
	mov	dx,ds
	mov	bx,es
	push	si
	push	di
	getarg	di,0			; get destination pointer
	getarg	es,1
	getarg	si,2			; get source pointer
	getarg	ds,3
	getarg	cx,4			; get number of bytes
	rep	movsb			; do it
	pop	di
	pop	si
	mov	es,bx
	mov	ds,dx
	pop	bp
	ret

_memcpy		endp


;
;**************************************************************************
;
; Compare memory.
; Input:  1. arg  -  far pointer to first string
;         2. arg  -  far pointer to second string
;         3. arg  -  number of bytes to compare
; Output: zero if equal, negative if 1. is lower, positive if 1. is higher
;
_memcmp		proc	near

	cld
	push	bp
	mov	bp,sp			; setup standard stack frame
	mov	dx,ds
	mov	bx,es
	push	si
	push	di
	getarg	di,0			; get destination pointer
	getarg	es,1
	getarg	si,2			; get source pointer
	getarg	ds,3
	getarg	cx,4			; get number of bytes
	repe	cmpsb
	jz	mcmp1
	lodsb				; compare first nonmatch byte
	cmp	al,es:[di]
	ja	mcmp2
	mov	ax,1
	jmp	short mcmp9

mcmp1:	xor	ax,ax
	jmp	short mcmp9		; setup correct return value
mcmp2:	mov	ax,-1
mcmp9:	pop	di
	pop	si
	mov	es,bx
	mov	ds,dx
	pop	bp
	ret

_memcmp		endp


;
;**************************************************************************
;
; Fill memory block with a single value
; Input:  1. arg  -  far pointer to destination
;         2. arg  -  value to fill memory with
;         3. arg  -  number of bytes to copy
; Output: none
;
_memset		proc	near

	cld
	push	bp
	mov	bp,sp			; setup standard stack frame
	mov	bx,es
	push	di
	getarg	di,0			; get destination pointer
	getarg	es,1
	getarg	ax,2			; get fill value
	getarg	cx,3			; get number of bytes
	rep	stosb			; do it
	pop	di
	mov	es,bx
	pop	bp
	ret

_memset		endp


;
;**************************************************************************
;
text_end

	end

