/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  netinit.c
 * Purpose: Initialize the network module
 * Entries: init_net, if_config
 *
 **************************************************************************
 *
 * Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "../headers/packet.h"
#include "../headers/net.h"
#include "../headers/romlib.h"
#include "../headers/general.h"
#include "./netpriv.h"
#include "./udp.h"
#include "./ip.h"
#include "./arp.h"
#include "./route.h"



/*
 **************************************************************************
 * 
 * Globale variables for the network library:
 *
 */
unsigned char     myhwaddr[ETH_ALEN];	/* my own hardware address	*/
         t_ipaddr myipaddr;		/* my own IP address		*/
         t_ipaddr mynetmask;		/* netmask for my network	*/
         t_ipaddr mybcastip;		/* broadcast IP address		*/
         char    *module_name;		/* name of init module		*/

/* Broadcast hardware address */
unsigned char bcasthw[ETH_ALEN] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };



/*
 **************************************************************************
 * 
 * Interface configuration: sets all IP addresses which are related to
 * a specific interface.
 */
void if_config(t_ipaddr myip, t_ipaddr netmask, t_ipaddr bcast)
{
  /* Adjust netmask if none given */
  if (netmask == IP_ANY) {
	if (IN_CLASS_A(myip))
		netmask = IP_CLASS_A;
	else if (IN_CLASS_B(myip))
		netmask = IP_CLASS_B;
	else if (IN_CLASS_C(myip))
		netmask = IP_CLASS_C;
  }

  /* Now set the interface values */
  myipaddr  = myip;
  mynetmask = netmask;
  mybcastip = bcast;
}



/*
 **************************************************************************
 * 
 * Network intialization:
 */
void init_net(void)
{
  /*
   * Setup default IP addresses. These are just assumed values, and should
   * later be replaced by the real values.
   */
#ifdef TESTADR
  myipaddr  = TESTADR;
#else
  myipaddr  = IP_LOCALHOST;
#endif
  mynetmask = IP_CLASS_C;
  mybcastip = IP_BROADCAST;

  /*
   * Initialize the different network layer modules. Note that we
   * can only get a hardware address from the packet driver, AFTER
   * at least one protocol has been initialized.
   */
  if (!init_route() || !init_arp() || !init_ip() || !init_udp() ||
      get_hw_addr((char *)myhwaddr, sizeof(myhwaddr)) != ETH_ALEN) {
	printf("\nERROR: cannot initialize module '%s'\n", module_name);
	fatal();
  }
}

