;**************************************************************************
;*
;* Boot-ROM-Code to load an operating system across a TCP/IP network.
;*
;* Module:  packet.asm
;* Purpose: Interface to the packet driver
;* Entries: _reg_type, _dereg_type, _write_packet, _get_mtu, _get_hw_addr,
;*          recvpkt
;*
;**************************************************************************
;*
;* Copyright (C) 1995 Gero Kuhlmann <gero@gkminix.han.de>
;*
;*  This program is free software; you can redistribute it and/or modify
;*  it under the terms of the GNU General Public License as published by
;*  the Free Software Foundation; either version 2 of the License, or
;*  any later version.
;*
;*  This program is distributed in the hope that it will be useful,
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*  GNU General Public License for more details.
;*
;*  You should have received a copy of the GNU General Public License
;*  along with this program; if not, write to the Free Software
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;*


;
;**************************************************************************
;
; Include assembler macros:
;
include ..\headers\asm\layout.inc
include ..\headers\asm\macros.inc
include ..\headers\asm\memory.inc
include .\pktpriv.inc


;
;**************************************************************************
;
; Define entry for packet type table:
;
type_entry	struc

type_id		dw	?		; ID for packet type
type_driver	dw	?		; pointer to type event routine
type_handle	dw	?		; packet driver handle for type

type_entry	ends


;
;**************************************************************************
;
; BSS segment:
;
bss_start

		public	bufsize, readbuf, writebuf, pktpoll

readtype	dw	RBUFNUM dup (?)	; type of packet in each buffer
readsize	dw	RBUFNUM dup (?)	; actual size of data in each buffer
readflag	dw	RBUFNUM dup (?)	; flag to indicate kept packet
readbuf		dw	?		; pointer to first read buffer
writebuf	dw	?		; pointer to write buffer
bufsize		dw	?		; size of each read buffer
currbuf		dw	?		; number of current read buffer
pktpoll		dw	?		; polling event handle
hwaddr		db	ETH_ALEN dup (?); hardware address
hwainit		db	?		; flag if hardware address initialized

typetab		type_entry	TYPENUM dup (<>)

bss_end


;
;**************************************************************************
;
; Start code segment.
;
text_start

	public	_reg_type		; define entry points
	public	_dereg_type
	public	_write_packet
	public	_get_mtu
	public	_get_hw_addr

	public	recvpkt

	extrn	_markpoll:near


;
;**************************************************************************
;
; Register a new packet type.
; Input:  1. arg  -  type identifier
;         2. arg  -  near pointer to poll callback routine
; Output: error code
;
_reg_type	proc	near

	push	bp
	mov	bp,sp			; setup standard stack frame
	push	si
	getarg	dx,0
	xor	si,si
	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
regt1:	mov	ax,type_id[bx]
	cmp	ax,dx			; is it already in list?
	jne	regt2
	mov	ax,-ERR_TYPE_INUSE
	jmp	short regt9
regt2:	or	ax,ax
	jnz	regt6			; is the slot available?
	or	si,si
	jnz	regt6			; save it, if we didn't get a free
	mov	si,bx			; slot already
regt6:	add	bx,size type_entry
	loop	regt1
	or	si,si			; did we find a free slot?
	jnz	regt3
	mov	ax,-ERR_BAD_TYPE	; no slot available
	jmp	short regt9

regt3:	mov	type_id[si],dx		; save type id
	getarg	ax,1			; get pointer to handler routine
	mov	type_driver[si],ax
	push	si
	push	es
	push	di
	lea	si,type_id[si]		; DS:SI  -  pointer to type ID
	mov	cx,size type_id		; CX     -  size of type ID
	mov	al,CL_DEFAULT		; AL     -  default class
	mov	bx,TYPE_DEFAULT		; BX     -  default type
	xor	dl,dl			; DL     -  default interface number
	push	cs
	pop	es			; ES:DI  -  pointer to receive handler
	mov	di,offset cgroup:receiver
	mov	ah,ACCESS_TYPE
	int	PKTINT			; setup access type
	pop	di
	pop	es
	pop	si
	jnc	regt4
regt5:	mov	al,dh			; in case of error prepare return
	xor	ah,ah			; value, e.g. error code
	neg	ax
	jmp	short regt9

regt4:	mov	type_handle[si],ax	; save access type handle
	test	hwainit,0FFh		; check if hardware address already
	jnz	regt7			; initialized
	push	es
	push	di
	mov	bx,ax
	mov	ax,ds
	mov	es,ax
	mov	di,offset dgroup:hwaddr
	mov	cx,length hwaddr	; get hardware address for current
	mov	ah,GET_ADDRESS		; handle from packet driver
	int	PKTINT
	pop	di
	pop	es
	jc	regt5
	inc	hwainit			; mark it as initialized
regt7:	xor	ax,ax			; return without error
regt9:	pop	si
	pop	bp
	ret

_reg_type	endp


;
;**************************************************************************
;
; Deregister a packet type.
; Input:  1. arg  -  type identifier
; Output: error code
;
_dereg_type	proc	near

	push	bp
	mov	bp,sp			; setup standard stack frame
	getarg	dx,0
	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
dereg1:	mov	ax,type_id[bx]
	cmp	ax,dx			; is the packet type known?
	je	dereg2
	add	bx,size type_entry
	loop	dereg1
	mov	ax,-ERR_BAD_TYPE	; packet type not found
	jmp	short dereg9

dereg2:	mov	type_id[bx],0		; clear table slot
	mov	bx,type_handle[bx]	; clear the type with the packet
	mov	ah,RELEASE_TYPE		; driver as well
	int	PKTINT
	jnc	dereg3
	mov	al,dh
	xor	ah,ah			; prepare error code as return
	neg	ax			; value
	jmp	short dereg9

dereg3:	xor	ax,ax			; return without error
dereg9:	pop	bp
	ret

_dereg_type	endp


;
;**************************************************************************
;
; Send a packet using the packet driver
; Input:  1. arg  -  near pointer to data buffer
;         2. arg  -  number of bytes in buffer
;         3. arg  -  packet type value
;         4. arg  -  near pointer to destination hardware address
; Output: error code
;
_write_packet	proc	near

	cld
	push	bp
	mov	bp,sp			; setup standard stack frame
	push	es
	push	si
	push	di
	mov	ax,ds
	mov	es,ax
	getarg	ax,2			; get packet type
	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
wrtp1:	cmp	ax,type_id[bx]		; find packet type in type table
	je	wrtp2
	add	bx,size type_entry
	loop	wrtp1
	mov	ax,-ERR_BAD_TYPE	; type not found
	jmp	short wrtp9

wrtp2:	mov	dx,type_handle[bx]	; determine packet driver handle
	mov	bx,writebuf
	mov	cx,ETH_ALEN
	lea	di,eth_source[bx]	; copy source hardware address into
	mov	si,offset dgroup:hwaddr	; send buffer
	rep	movsb

	mov	cx,ETH_ALEN
	lea	di,eth_dest[bx]		; get pointer to destination hardware
	getarg	si,3			; address
	rep	movsb			; copy destination address
	getarg	ax,2			; copy packet type
	mov	eth_type[bx],ax

	lea	di,ETH_HLEN[bx]		; compute send buffer address
	getarg	cx,1			; get number of characters to write
	getarg	si,0			; get data pointer
	cmp	cx,ETH_DATA_MAX
	jbe	wrtp4			; check if maximum number of data
	mov	cx,ETH_DATA_MAX		; bytes exceeded
wrtp4:	mov	ax,cx
	rep	movsb			; copy payload into output buffer
	cmp	ax,ETH_DATA_MIN
	jae	wrtp5			; check if minimum number of data
	mov	cx,ETH_DATA_MIN		; bytes copied
	sub	cx,ax
	xor	al,al			; pad with zeroes if not
	rep	stosb
	mov	ax,ETH_DATA_MIN

wrtp5:	mov	si,bx			; set pointer to write buffer
	mov	bx,dx			; set packet driver handle
	mov	cx,ax
	add	cx,ETH_HLEN		; set packet buffer size
	mov	ah,SEND_PKT		; now send the packet
	int	PKTINT
	jnc	wrtp8
	mov	al,dh
	xor	ah,ah			; return with error
	neg	ax
	jmp	short wrtp9

wrtp8:	xor	ax,ax			; no error
wrtp9:	pop	di
	pop	si
	pop	es
	pop	bp
	ret

_write_packet	endp


;
;**************************************************************************
;
; Return Maximum Transfer Unit.
; Input:  none
; Output: MTU
;
_get_mtu	proc	near

	mov	ax,bufsize
	sub	ax,ETH_HLEN		; MTU is buffer size minus header
	ret				; length

_get_mtu	endp


;
;**************************************************************************
;
; Return hardware address.
; Input:  1. arg  -  near pointer to buffer
;         2. arg  -  length of buffer
; Output: error code if negative, else size of hardware address
;
_get_hw_addr	proc	near

	cld
	push	bp
	mov	bp,sp			; setup standard stack frame
	push	es
	push	si
	push	di
	mov	ax,ds
	mov	es,ax
	mov	ax,-ERR_NO_SPACE
	getarg	cx,1
	cmp	cx,ETH_ALEN		; check that destination buffer is
	jb	gethw9			; large enough

	getarg	di,0
	mov	si,offset dgroup:hwaddr
	mov	cx,ETH_ALEN		; just copy the hardware address
	rep	movsb
	mov	ax,ETH_ALEN		; return number of bytes in
gethw9:	pop	di			; address
	pop	si
	pop	es
	pop	bp
	ret

_get_hw_addr	endp


;
;**************************************************************************
;
; This routine gets called by the packet driver when a new packet arrives.
; Note that this routine cannot use any segment registers, because it gets
; called from within a hardware interrupt.
; Input:  AX  -  function number: 0 = return buffer pointer, 1 = buffer filled
;         BX  -  handle
;         CX  -  length of data received (only when AX = 1)
;         DS:SI  -  pointer to receive buffer (only when AX = 1)
; Output: CX     -  length of buffer (only when AX = 0)
;         ES:DI  -  pointer to receive buffer (only when AX = 0)
; Registers changed: AX, BX, CX, DX, ES, SI, DI
;
receiver	proc	far

	push	ds
	mov	dx,NEWDATA
	mov	ds,dx
	or	ax,ax			; check function code
	jnz	rec3

; Return address of receive buffer in ES:DI. currbuf is the number of
; the next buffer to be read by the upper level routines. From this
; buffer on search for the next free buffer. Discard the packet if
; there is no such free buffer.

	mov	es,ax
	mov	di,ax			; discard packet if no free buffer
	mov	bx,currbuf		; search for first free buffer
	shl	bx,1
rec1:	mov	ax,readsize[bx]
	or	ax,ax			; free buffer?
	jz	rec2
	add	bx,2			; check next buffer
	cmp	bx,RBUFNUM * 2
	jb	rec8
	xor	bx,bx			; wrap around
rec8:	cmp	bx,currbuf
	je	rec9
	jmp	rec1

rec2:	mov	ax,bufsize
	mov	cx,ax
	shr	bx,1			; compute offset of read buffer
	mul	bx
	add	ax,readbuf
	mov	di,ax
	mov	ax,ds
	mov	es,ax
	jmp	short rec9

; The buffer has been read. Now mark it as busy, and notify the polling
; routine that there is something to do.

rec3:	mov	ax,si
	mov	si,offset dgroup:typetab
	mov	dx,length typetab
rec4:	cmp	bx,type_handle[si]	; find handle in type table
	je	rec5
	add	si,size type_entry
	dec	dx
	jnz	rec4			; discard packet if handle not found
	jmp	short rec9

rec5:	mov	dx,type_id[si]
	pop	bx
	push	bx			; data segment of buffer must be
	cmp	bx,NEWDATA		; valid
	jne	rec9			; otherwise discard packet

	xor	bx,bx			; determine number of read buffer
	sub	ax,readbuf		; by scanning through buffer list
rec6:	jz	rec7			; until the buffer address found
	inc	bx
	cmp	bx,RBUFNUM
	jae	rec9			; buffer not found?
	sub	ax,bufsize		; check for next buffer
	jmp	short rec6

rec7:	shl	bx,1
	mov	readsize[bx],cx		; save buffer size and mark buffer
	mov	readtype[bx],dx
	push	pktpoll			; notify polling routine of work
	call	_markpoll
	pop	ax

rec9:	pop	ds
	ret

receiver	endp


;
;**************************************************************************
;
; Get a received packet out of the buffer list, and call the upper layer
; handler. This routine gets called by the polling service. The upper
; layer handler gets the following parameters on it's stack:
;	char *buf    -  pointer to receive buffer
;	int bufsize  -  number of data bytes in receive buffer
;	int typeval  -  type of received packet
;	char *addr   -  pointer to sender's address
; If the return value is non-zero, the packet can get discarded. Other-
; wise it must be kept in the buffer.
; Input:  none
; Output: AX  -  non-zero, if polling request not to be reset
; Registers changed: AX, BX, CX, DX
;
recvpkt		proc	near

	xor	bx,bx
	xchg	bx,currbuf
	shl	bx,1			; scan through all input buffers
	mov	dx,bx
recv1:	mov	ax,readsize[bx]		; continue with next buffer, if current
	or	ax,ax			; one is empty
	jnz	recv2
recv6:	add	bx,2			; advance to next buffer
	cmp	bx,RBUFNUM * 2
	jb	recv8			; wrap around ?
	xor	bx,bx
recv8:	cmp	bx,dx			; check if at end of list
	je	recv9
	jmp	recv1

recv2:	push	bx
	push	dx
	mov	cx,si
	mov	si,bx			; save buffer number for later
	shr	bx,1
	mov	ax,bufsize		; compute pointer to input buffer
	mul	bx
	add	ax,readbuf
	mov	bx,ax
	push	readflag[si]		; push buffer reuse flag
	lea	ax,eth_source[bx]	; push pointer to source hardware
	push	ax			; address
	mov	dx,readtype[si]
	push	dx			; push packet type
	mov	ax,readsize[si]
	sub	ax,ETH_HLEN
	push	ax			; push buffer size (without header)
	add	bx,ETH_HLEN
	push	bx			; push buffer pointer (without header)
	mov	si,cx

	mov	ax,1
	mov	bx,offset dgroup:typetab
	mov	cx,length typetab
recv3:	cmp	dx,type_id[bx]
	je	recv4
	add	bx,size type_entry	; search type in type list
	loop	recv3
	jmp	short recv5

recv4:	mov	ax,1
	mov	bx,type_driver[bx]	; call type handler
	or	bx,bx
	jz	recv5
	call	bx
recv5:	add	sp,10			; cleanup stack
	pop	dx
	pop	bx
	or	ax,ax			; don't touch the packet
	jz	recv7
	mov	readsize[bx],0		; discard packet
	mov	readflag[bx],0
	jmp	recv6			; continue with next buffer

recv7:	inc	readflag[bx]		; increase reuse flag
	test	currbuf,0FFFFh		; if this is the first buffer, which
	jnz	recvB			; has to be left, save it's pointer
	mov	ax,bx
	shr	ax,1
	mov	currbuf,ax
recvB:	jmp	recv6

recv9:	mov	bx,currbuf
	shl	bx,1			; if there is at least one buffer kept
	mov	ax,readsize[bx]		; we cannot reset the poll flag
	ret

recvpkt		endp


;
;**************************************************************************
;
text_end

	end

