#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#ifdef MSDOS
#include <io.h>
#endif


#undef DEBUG


#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif


#define BLKSIZE		512
#define ROMSIG		0xAA55
#define EXESIG		0x5A4D
#define MAXARGS		126
#define DATAOFS		8

#define PKTHEADER	5
#define COMPR_NONE	1
#define COMPR_STD	2
#define COMPR_SIG	0x4B47


char *progname;
char *romname = NULL;
char *pktargs = NULL;
char *pktname = NULL;
char *outname = NULL;
int infile;
int outfile;



int dowrite(unsigned char *buf, int bufsize)
{
  int i, ofs;

  ofs = 0;
  while (TRUE) {
	if ((i = write(outfile, (char *)(&buf[ofs]), bufsize - ofs)) < 0) {
		perror(outname);
		exit(1);
	}
	ofs += i;
	if (i == 0 || ofs >= bufsize) break;
  }
#ifdef DEBUG
  fprintf(stderr, "w:%d<%s>\n", ofs, outname);
#endif
  return ofs;
}



int doread(unsigned char *buf, int bufsize, char *name)
{
  int i, ofs;

  ofs = 0;
  while (TRUE) {
	if ((i = read(infile, (char *)(&buf[ofs]), bufsize - ofs)) < 0) {
		perror(name);
		exit(1);
	}
	ofs += i;
	if (i == 0 || ofs >= bufsize) break;
  }
#ifdef DEBUG
  fprintf(stderr, "r:%d<%s>\n", ofs, name);
#endif
  return ofs;
}



void usage(void)
{
  fprintf(stderr, "usage: %s -r<rom file> -p<packet driver> ", progname);
  fprintf(stderr, "-a<pktdrv args> [-o<out file>]\n");
  exit(1);
}



main(int argc, char **argv)
{
  int romimage;
  int argofs;
  int pktofs;
  int writecnt;
  int romlength;
  unsigned char inbuf[BLKSIZE];
  char *cp;
  int i, len;

  /* Determine our own name */
#ifdef MSDOS
  if ((cp = strrchr(argv[0], '\\')) == NULL)
#else
  if ((cp = strrchr(argv[0], '/')) == NULL)
#endif
	cp = argv[0];
  else
	cp++;
  progname = strdup (cp);

  /* Isolate the different command line options */
  for (i = 1; i < argc; i++) {
	if (*argv[i] != '-' || strlen(argv[i]) < 2)
		usage();
	 switch (argv[i][1]) {
		case 'r':  if (strlen(argv[i]) > 2) {
				cp = &argv[i][2];
		           } else {
				if (++i >= argc) usage();
				cp = argv[i];
		           }
		           romname = strdup(cp);
		           break;
		case 'p':  if (strlen(argv[i]) > 2) {
				cp = &argv[i][2];
		           } else {
				if (++i >= argc) usage();
				cp = argv[i];
		           }
		           pktname = strdup(cp);
		           break;
		case 'o':  if (strlen(argv[i]) > 2) {
				cp = &argv[i][2];
		           } else {
				if (++i >= argc) usage();
				cp = argv[i];
		           }
		           outname = strdup(cp);
		           break;
		case 'a':  if (strlen(argv[i]) > 2) {
				cp = &argv[i][2];
		           } else {
				if (++i >= argc) usage();
				cp = argv[i];
		           }
		           pktargs = strdup(cp);
		           break;
		default:   usage();
	}
  }

  /* Check if all necessary command line options are given */
  if (romname == NULL || pktname == NULL || pktargs == NULL)
	usage();
  if (strlen(pktargs) > MAXARGS) {
	fprintf(stderr, "%s: packet driver argument too long\n", progname);
	exit(1);
  }

  /* Open the output file */
  if (outname == NULL) {
	outfile = fileno(stdout);
  } else {
#ifdef MSDOS
	if ((outfile = open(outname, O_CREAT | O_WRONLY | O_BINARY, 0666)) < 0) {
#else
	if ((outfile = open(outname, O_CREAT | O_WRONLY, 0666)) < 0) {
#endif
		perror(outname);
		exit(1);
	}
  }

  /* Open the input file, read the first block, and check type of file.
   * If the input file is a boot floppy image, copy the first block directly
   * to the output file, because it is the floppy boot sector.
   */
  len = BLKSIZE;
#ifdef MSDOS
  if ((infile = open(romname, O_RDONLY | O_BINARY)) < 0) {
#else
  if ((infile = open(romname, O_RDONLY)) < 0) {
#endif
	perror(romname);
	exit(1);
  }
  if (doread(inbuf, len, romname) != len) {
	fprintf(stderr, "%s: unexpected end of file\n", romname);
	exit(1);
  }
  romimage = *((unsigned short *)(&inbuf[0])) == ROMSIG ? TRUE : FALSE;
  if (!romimage) {
	(void) dowrite(inbuf, len);
	if (doread(inbuf, len, romname) != len) {
		fprintf(stderr, "%s: unexpected end of file\n", romname);
		exit(1);
	}
	if (*((unsigned short *)(&inbuf[0])) != ROMSIG) {
		fprintf(stderr, "%s: invalid boot rom image\n", romname);
		exit(1);
	}
  }
  romlength = *((unsigned short *)(&inbuf[DATAOFS + 4])) + 16;

  /* Now copy the rom image file into the output file. While writing to
   * the output file, count the number of bytes written, so that the offset
   * to the packet driver can be determined. Also only copy the relevant
   * bytes excluding the BSS segment, which is only zero bytes anyway.
   */
  writecnt = 0;
  romlength -= len;
  while (TRUE) {
	writecnt += dowrite(inbuf, len);
	if (romlength <= 0) break;
	if ((len = doread(inbuf, BLKSIZE, romname)) == 0) break;
	if (len > romlength)
		len = romlength;
	romlength -= len;
  }
  argofs = writecnt;

  /* Next write the packet driver argument line to the output file. The
   * argument string contains a leading byte which gives the number of
   * characters in the string.
   */
  len = strlen(pktargs);
  inbuf[0] = len & 0xFF;
  strncpy((char *)(&inbuf[1]), pktargs, len);
  len += 2;
  writecnt += dowrite(inbuf, len);
  pktofs = writecnt;

  /* Now copy the packet driver into the output file, but retain some
   * space for the packet driver header.
   */
  len = BLKSIZE - PKTHEADER;
#ifdef MSDOS
  if ((infile = open(pktname, O_RDONLY | O_BINARY)) < 0) {
#else
  if ((infile = open(pktname, O_RDONLY)) < 0) {
#endif
	perror(pktname);
	exit(1);
  }
  len = doread(&inbuf[PKTHEADER], len, pktname) + PKTHEADER;
  if (*((unsigned short *)(&inbuf[PKTHEADER])) == EXESIG) {
	fprintf(stderr, "%s: cannot read EXE type files\n", pktname);
	exit(1);
  }
  writecnt = 0;
  while (TRUE) {
	writecnt += dowrite(inbuf, len);
	if ((len = doread(inbuf, BLKSIZE, pktname)) == 0) break;
  }
  writecnt -= PKTHEADER;

  /* Finally write the packet driver offsets and the packet driver header
   * into the destination file.
   */
  i = romimage ? DATAOFS : DATAOFS + BLKSIZE;
  if (lseek(outfile, (long)i, 0) < 0) {
	perror(outname);
	exit(1);
  }
  *((short *)(&inbuf[0])) = pktofs;
  *((short *)(&inbuf[2])) = argofs;
  (void)dowrite(inbuf, 4);

  i = romimage ? pktofs : pktofs + BLKSIZE;
  if (lseek(outfile, (long)i, 0) < 0) {
	perror(outname);
	exit(1);
  }
  *((short *)(&inbuf[0])) = COMPR_SIG;
  *((short *)(&inbuf[2])) = writecnt;
  *((unsigned char *)(&inbuf[4])) = COMPR_NONE;
  (void)dowrite(inbuf, 5);

  close(outfile);
  exit(0);
}

